#ifndef IMAGEWINDOW_CPP
#define IMAGEWINDOW_CPP
#include <qregexp.h>
#include <strings.h>
#include <qtimer.h>
#include <qevent.h>
#include <iostream.h>
#include <fstream.h>
#include <tiffio.h>
#include <math.h>
#include <qlayout.h>
#include "imagewindow.h"
#include "imagewindow.moc"
#include "imageloader.h"

//********************************8
//construcot
// loads an emtpy image as the background and creates the window
ImageWindow::ImageWindow(QWidget * parent, const char * name)
    : QWidget(parent, name)
{
	//setSize( 400, 400 );
	setBackgroundColor( white );
	imageLoader = new ImageLoader( this );
	image = new QImage;
	image->create( 32, 32, 32 );
	image->fill( qRgb( 0,0,0) );
   	viewrect.setCoords(0,0, (image->size()).width() , (image->size()).height() );
   	setMinimumSize( 600, 600 );
	maxWindows = 0;
	showWindows = false;
	replayWindows = 0;
	mode = RECTANGLES;
    fixedWidth = 50;
    fixedHeight = 50;
}

void ImageWindow::zoomin()
{
	//set the viewing window to be the selection rect
	viewrect = rect.last();
	//create a new pix to dislpay that is whats inside that window
	setPaletteBackgroundPixmap( ( image->copy(viewrect) ).smoothScale( size() ) );
	//display it
	update();
}					

void ImageWindow::zoomout()
{
	//reset viewing window size
	viewrect.setCoords(0,0, (image->size()).width() , (image->size()).height() );
	//make a new pix to dislpay
	setPaletteBackgroundPixmap( image->copy(viewrect).smoothScale( size() ) );
	//display it
	update();
}

void ImageWindow::paintEvent( QPaintEvent * e)
{
  QRect paintRect = e->rect();
  QPainter paint ( this );  
  paint.setPen( white );
  int x0, y0, x, y;
	//decide where ot paint the rectangle
  int i;
  for(i = 0; i < rect.count(); i ++) 
  {
	  translateInv(rect[i].left(),  rect[i].top(), x0, y0);
	  translateInv(rect[i].right(), rect[i].bottom(), x, y);
	//paint it
	  paint.drawRect( QRect( x0, y0, x-x0, y-y0 ) );
	  
  }
  for( i = 0; i < point.count(); i ++ )
  {
	  translateInv(  point[i].x(), 	point[i].y(), x, y );
	  paint.drawEllipse( x-5, y-5, 10, 10 );
  }
}

//converts from window coordinates to image coordinates
void ImageWindow::translate( int x0, int y0, int & x, int & y)
{
	x = (int)( x0 / (float)(size()) .width()  * viewrect.width()  + viewrect.x() );
	y = (int)( y0 / (float)(size()) .height() * viewrect.height() + viewrect.y() );
}
//converts from image coordinates back to window coordinates
void ImageWindow::translateInv( int x, int y, int & x0, int & y0)
{
	x0 = (int)( (x - viewrect.x() ) / ((float) viewrect. width() ) * (size()). width() );
	y0 = (int)( (y - viewrect.y() ) / ((float) viewrect.height() ) * (size()).height() );
}	

//mouse move only called if mouse is pressed, so draw rectangle to follow movement
void ImageWindow::mouseMoveEvent(QMouseEvent * e)
{ 
	if( !drawing ) return;
	int x0, y0, x, y, tmp;
    int a, b, c, d;
	
	switch( mode ){ 
		case( POINTS ):
			translate( e->x(), e->y(), x, y );

			translateInv( point.last().x(), point.last().y(), x0, y0 );
			erase( QRect( x0-7,y0-7,15,15));
					
			point.last().setX( x );
			point.last().setY( y );
			break;
			
        case( FIXEDRECTANGLES ):
			translateInv( rect.last().left(),  rect.last().top(), a, b );
			translateInv( rect.last().right(), rect.last().bottom(), c, d );
			//erase the old rectangle
			erase( QRect( QPoint(a-1,b-1), QPoint(a+1,d+1) ));
			erase( QRect( QPoint(a-1,b-1), QPoint(c+1,b+1) ));
			erase( QRect( QPoint(c-1,b-1), QPoint(c+1,d+1) ));
			erase( QRect( QPoint(a-1,d-1), QPoint(c+1,d+1) ));

            translate( e->x(), e->y(), x,y );
            rect.last().setRect( x-fixedWidth/2, y-fixedHeight/2, fixedWidth, fixedHeight );

            break;
		case( RECTANGLES ):
			translate(posLastClicked.x(), posLastClicked.y(), x0, y0);	
			translate(e->x(), e->y(), x, y);
			if(x < x0){
				tmp = x0;
				x0 = x;
				x = tmp;
			}
			if(y < y0){
				tmp = y0;
				y0 = y;
				y = tmp;
			}
			
			translateInv( rect.last().left(),  rect.last().top(), a, b );
			translateInv( rect.last().right(), rect.last().bottom(), c, d );
			//erase the old rectangle
			erase( QRect( QPoint(a-1,b-1), QPoint(a+1,d+1) ));
			erase( QRect( QPoint(a-1,b-1), QPoint(c+1,b+1) ));
			erase( QRect( QPoint(c-1,b-1), QPoint(c+1,d+1) ));
			erase( QRect( QPoint(a-1,d-1), QPoint(c+1,d+1) ));
			rect.last().setCoords(x0,y0,x+1,y+1);
			//do NOT clear the window before redrawing, paint over what is there
			break;
	}
	repaint( false );
}

//when mouse pressed start a rectangle
void ImageWindow::mousePressEvent(QMouseEvent * e)
{
	posLastClicked = e->pos();
	drawing = true;
	
	switch( e->button() ){
		case( QMouseEvent::LeftButton ):
			switch( mode ){
				case( POINTS ):			point.push( QPoint() );		break;
                case( FIXEDRECTANGLES ):
                case( RECTANGLES ) :  	rect.push ( QRect() );		break;
            
			}

		case( QMouseEvent::RightButton ):
		  switch( mode ){
			  case( POINTS ): 		if( point.count() == 0 )	point.push( QPoint() ); 	break;
              case( FIXEDRECTANGLES ):
			  case( RECTANGLES ): 	if( rect. count() == 0 )   	rect. push( QRect () );     break;
		  }
		  break;
	}
	mouseMoveEvent( e );
}

void ImageWindow::clearrect()
{
	rect.last().setCoords( -1,-1,-1,-1 );
	emit( rectChanged( rect ) );
	emit( pointChanged( point ) );
}

//add the rectangle to its stats when mouse is released
void ImageWindow::mouseReleaseEvent(QMouseEvent * e)
{ 
	emit( rectChanged( rect ) );
	emit( pointChanged( point )) ;
	drawing = false;
    update();
}

//zooming is lost when window is resized
void ImageWindow::resizeEvent(QResizeEvent * e)
{
	setPaletteBackgroundPixmap( image->copy( viewrect ).smoothScale( size() ) );
}
  
//opens some file filename
void ImageWindow::openFrom(QString filename, QString nextfile, bool hasnext ) 
{
  setPaletteBackgroundPixmap( *imageLoader->get( filename ) ); // tell the loader to grab the file
  image = imageLoader->getImage();
  if( hasnext )
	  imageLoader->open( nextfile );
  viewrect.setCoords(0,0, (image->size()).width() , (image->size()).height() );
  rect.clear();
  point.clear();
  update();
}

//the following two are for any future algorithms, they are already linked to the menu and have full access everything you would need here

void ImageWindow::showReplayWindows()
{
	qWarning("showing replay windows");
	showWindows = true;
	emit( getMaxWindows() );
}
	
void ImageWindow::setMaxWindows( int _maxWindows )
{
    QWidget* replayWindow = new QWidget( 0, "replayWindow", Qt::WStyle_StaysOnTop );
    replayWindow->setCaption("Selected Objects");
	maxWindows = _maxWindows;
    int s = ((int)(sqrt(maxWindows)))+1;
    qWarning("%i windows -> size %i x %i", maxWindows, s, s );
    maxWindows = s*s;
    replayWindow->show();
    QGridLayout* grid = new QGridLayout( replayWindow, s, s, 2,5,"replayLayout");
	replayWindows = new QLabel*[maxWindows];
	int i;
	for(i = 0; i < maxWindows; i ++ ){
		replayWindows[i] = new QLabel( replayWindow, QString("rectangle #%1").arg(i) );
        replayWindows[i]->show();
        replayWindows[i]->setMinimumSize(0,0);
        grid->addWidget( replayWindows[i],  i/s, i%s );
        replayWindows[i]->setScaledContents(true);
	}
}

void ImageWindow::update()
{
  if( showWindows ) {
	  int i, j;
      j = 0;
      int ws = 30;
      int w2 = ws/2;
      static QPixmap* blank = 0;
      if( !blank ){
          blank = new QPixmap( 32,32 );
          blank->fill( black );
      }
      for( i = 0; i < point.count() && i < maxWindows; i ++ )
          replayWindows[j++]->setPixmap( image->copy( QRect( point[i].x() - w2, point[i].y() - w2,ws,ws )).smoothScale( replayWindows[i]->size()));
      
	  for(i = 0; i < rect.count() && i + j < maxWindows; i ++ )
		  replayWindows[j++]->setPixmap( image->copy( rect[i] ).smoothScale( replayWindows[i]->size() ) );

      for(; j < maxWindows; j ++ )
		  replayWindows[j++]->setPixmap( *blank );
  }
  QWidget::update();
}


#include "classifier.moc"
void ImageWindow::classify()
{
    QImage i = image->copy();
	cout << "ImageWindow::classify()" << endl;
    Classifier* c = new Classifier( 0, "Chris's Quadtree building algorithm", i );
    c->show();
}
void ImageWindow::detect()
{
	cout << "ImageWindow::detect()" << endl;
}

#endif
