#include <qimage.h>
#include <tiffio.h>

void read_tiff_image(QImageIO *iio)
{
  TIFF *inimg;
  QImage outimg;

  iio->setStatus(1);            // reset to 0 if all goes well

  inimg = TIFFOpen(iio->fileName(), "r");
  if (!inimg)   {               // failed to open file
     TIFFError(__FUNCTION__, "Failed to open tif file");
     return;
  }

  // read pixels.
  uint32 width, height;
  uint16 bitspersample;

  TIFFGetField(inimg, TIFFTAG_IMAGEWIDTH, &width);
  TIFFGetField(inimg, TIFFTAG_IMAGELENGTH, &height);
  TIFFGetField(inimg, TIFFTAG_BITSPERSAMPLE, &bitspersample);

  if (bitspersample == 1)
  {
     outimg.create(width, height, 1, 2, QImage::BigEndian);
     if (outimg.isNull()) 
     {
        TIFFClose(inimg);
        return;
     }
     outimg.setColor(0, qRgb(0,0,0));
     outimg.setColor(1, qRgb(255,255,255));
     
     uchar** lines = outimg.jumpTable();                
     for (uint row = 0; row < height; row++) 
     {   
        if (TIFFReadScanline(inimg, lines[row], row, 0) < 0)
        {
           TIFFClose(inimg);
           TIFFError(__FUNCTION__, "Impossible de lire les lignes");
           return;      
        }  
    }

    TIFFClose(inimg);
    iio->setStatus(0);         
    iio->setImage(outimg);
    return;
  }
 
  TIFFRGBAImage img;
  char emsg[1024];

  size_t npixels = width * height;

  if (TIFFRGBAImageBegin(&img, inimg, 1, emsg)) 
  {
     img.orientation = ORIENTATION_BOTLEFT;
     uint32 *raster = (uint32*) _TIFFmalloc(npixels * sizeof (uint32));
     if (raster != NULL)
        if (TIFFRGBAImageGet(&img, raster, width, height))
        {
           outimg.create(width, height, 32);
           if (outimg.isNull()) 
           {
              TIFFClose(inimg);
              return;
           }
      
           memcpy(outimg.bits(), raster, npixels * sizeof (uint32));

           _TIFFfree(raster);
           TIFFRGBAImageEnd(&img);
        }
     else 
     {
        TIFFError(__FUNCTION__, "Memory Allocation Failed");
        TIFFClose(inimg);
        return;
     }
  } 
  else 
  {
     TIFFError(__FUNCTION__, emsg);
     TIFFClose(inimg);
     return;
  }

  TIFFClose(inimg);
  iio->setStatus(0);           
  iio->setImage(outimg);
}

void write_tiff_image(QImageIO *iio)
{
   iio->setStatus(1);  // KO
}

void initTiffHandler()
{
  // initialize load routines
  QImageIO::defineIOHandler("TIFF", "^[MI][MI]", 0, read_tiff_image, write_tiff_image);
  QImageIO::defineIOHandler("TIF", "^[MI][MI]", 0, read_tiff_image, write_tiff_image);
}

