#include <qapplication.h>
#include <qspinbox.h>
#include <qpushbutton.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qlayout.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qfiledialog.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qvaluestack.h>
#include "window.h"
#include "window.moc"
#include "faceswidget.cpp"
#include "vehicleswidget.cpp"
#include "imagewindow.h"
#include "statmanager.h"


/** initializes all the main options\, insantiates all the major objects and connects everything up **/
Window::Window(QWidget* parent, const char* name, int argc, char ** argv, QApplication* app )
    : QMainWindow(parent,name)
{
    setCaption("Image Designator");
    setFocusPolicy( QWidget::StrongFocus );
    connect( app, SIGNAL( aboutToQuit()), this, SLOT( aboutToQuit()));
/*************************************************************************************/

    QWidget* panel = new QWidget( this, "Panel" );
    panel->setFocusPolicy( QWidget::StrongFocus );
	ImageWindow * imageWindow = new ImageWindow( panel, "imageWindow" );
    setCentralWidget( panel );
    currentFile 		= new Label( panel , "current file");
    currentDirectory 	= new Label( panel , "current directory");
/*************************************************************************************/ 
    statManager = new StatManager( 0, "statmanager"); // ( parent widget, name of widget )
   connect(statManager,SIGNAL(openFrom(QString,QString,bool)), imageWindow, SLOT(openFrom(QString, QString, bool)) );  // ( class emitting, signal emittied, class listening, function connected too )
   connect(statManager,SIGNAL(filenameChanged(QString)), currentFile, SLOT(setText (QString)));  // ( class emitting, signal emittied, class listening, function connected too )
   connect(statManager,SIGNAL(directoryChanged(QString)), currentDirectory, SLOT(setText (QString)));  // ( class emitting, signal emittied, class listening, function connected too )
   connect(statManager,SIGNAL(dataChanged(int, int)),		this, SLOT(setData(int, int)));
   connect(statManager,SIGNAL(rectChanged( QValueStack<QRect> )),imageWindow, SLOT(setRect( QValueStack <QRect> )));  
   connect(statManager,SIGNAL(pointChanged( QValueStack<QPoint> )),imageWindow, SLOT(setPoint( QValueStack <QPoint> )));  
   connect(imageWindow ,SIGNAL(getMaxWindows() ), statManager, SLOT( getMaxWindows()) );
   connect(statManager, SIGNAL(setMaxWindows(int) ), imageWindow, SLOT( setMaxWindows(int)) );

   connect(statManager, SIGNAL(openReplays() ), imageWindow, SLOT( showReplayWindows() ));

   connect(this, SIGNAL(next()),               statManager, SLOT(next()));      
   connect(this, SIGNAL(openFrom(QStringList)),statManager, SLOT(openImages(QStringList)));
   connect(this, SIGNAL(openImageList(QString)),statManager, SLOT(openImageList(QString)));
   connect(this, SIGNAL(openingDir(QString)),  imageWindow, SLOT(setwd(QString)));
   connect(this, SIGNAL(openingDir(QString)),  statManager, SLOT(openDir(QString)));
   //connect(this, SIGNAL(setRect(QValueStack<QRect>)),       statManager, SLOT(setrect(QValueStack<QRect>)));
   //connect(this, SIGNAL(setRect(QValueStack<QRect>)),       imageWindow, SLOT(setRect(QValueStack<QRect>)));
   connect(imageWindow, SIGNAL(rectChanged(QValueStack<QRect> )), statManager, SLOT(setrect( QValueStack<QRect> )));
   connect(imageWindow, SIGNAL(pointChanged(QValueStack<QPoint> )), statManager, SLOT(setpoint( QValueStack<QPoint> )));
     
/*************************************************************************************/
facesWidget    = new FacesWidget   ( panel, "facesWidget", this); 
   connect(facesWidget->ageButtonGroup,        SIGNAL(clicked(int)),  statManager, SLOT(setage(int)));  
   connect(facesWidget->expressionButtonGroup, SIGNAL(clicked(int)),  statManager, SLOT(setexpression(int)));
   connect(facesWidget->genderButtonGroup,     SIGNAL(clicked(int)),  statManager, SLOT(setgender(int)));
   connect(facesWidget->glassesButtonGroup,    SIGNAL(clicked(int)),  statManager, SLOT(setglasses(int))); 
   connect(facesWidget->angleButtonGroup,      SIGNAL(clicked(int)),  statManager, SLOT(setangle(int)));
   connect(facesWidget->raceButtonGroup,       SIGNAL(clicked(int)),  statManager, SLOT(setrace(int)));

vehiclesWidget = new VehiclesWidget( panel, "facesWidget", this);
   connect(vehiclesWidget->brightnessButtonGroup, SIGNAL(clicked(int)), statManager, SLOT(setbrightness(int)));
   connect(vehiclesWidget->complexityButtonGroup, SIGNAL(clicked(int)), statManager, SLOT(setcomplexity(int)));
   connect(vehiclesWidget->typeButtonGroup,       SIGNAL(clicked(int)), statManager, SLOT(settype(int)));   

   none();
/*************************************************************************************/
QGridLayout *mainLayout = new QGridLayout( panel,2,3,0,0,"mainlayout");     
   mainLayout->addMultiCellWidget(         imageWindow, 	0,1,0,1);
   mainLayout->addWidget(                  facesWidget,					0,  2);
   mainLayout->addWidget(                  vehiclesWidget,				0,  2);
   mainLayout->addWidget(                  currentDirectory,			2,  0 );
   mainLayout->addMultiCellWidget(         currentFile,		2,2,1,2 );
   mainLayout->setColStretch(0,1);
   mainLayout->setRowStretch(0,1);

QPopupMenu *filemenu = new QPopupMenu(this);
   filemenu->insertItem("Load all images in a Directory",	this,        SLOT(openDir()), CTRL+Key_O );
   filemenu->insertItem("Open Images",                   	this,        SLOT(open()) );
   filemenu->insertItem("Load a from a list of images/dat",	this,     	 SLOT(openListOfImages()) );
//   filemenu->insertItem("Clear List of Files",    statManager,        SLOT(clearAll()) );
   filemenu->insertItem("Replay changes", 			 statManager, 		 SLOT( replay() ) );
 //  filemenu->insertItem("Rebuild Datafiles",      statManager,        SLOT(saveAll()), CTRL+Key_S);
   filemenu->insertItem("Exit",                          	app,        SLOT(quit()),    CTRL+Key_Q );    

QPopupMenu *nextmenu = new QPopupMenu( this );
   nextmenu->insertItem("Next Undefined Image",   statManager,        SLOT(next()),    Key_Space);
   nextmenu->insertItem("Forward",                statManager,        SLOT(forward()), Key_Return);
   nextmenu->insertItem("Backwards",              statManager,        SLOT(back()),    Key_Backspace);

QPopupMenu* selectmenu = new QPopupMenu( this );
	selectmenu->insertItem("Select Points", 	imageWindow, SLOT( setModePoint()) );
	selectmenu->insertItem("Select Rectangles", imageWindow, SLOT( setModeRect()) );
	selectmenu->insertItem("Select Fixed Rectangles", imageWindow, SLOT( setModeFixedRect()) );

QPopupMenu *editmenu = new QPopupMenu(this);
//   editmenu->insertItem("Undo",  this,        SLOT(undo()) , CTRL+Key_Z);
   editmenu->insertItem("Clear Settings",  statManager, SLOT(clear()), Key_Delete );
   //editmenu->insertItem("Clear Rectangle", imageWindow, SLOT(clearrect()), Key_Insert );
   editmenu->insertItem("Erase last Rectangle", statManager, SLOT( popRect()));
   editmenu->insertItem("Erase last Point", statManager, SLOT( popPoint()));

QPopupMenu *optionmenu = new QPopupMenu(this);
   optionmenu->insertItem("Classify", imageWindow,    SLOT(classify()));
   optionmenu->insertItem("Detect",   imageWindow,    SLOT(detect()));

QPopupMenu * viewmenu = new QPopupMenu( this );
   viewmenu->insertItem("Zoom In",     imageWindow,    SLOT(zoomin()), Key_Z);
   viewmenu->insertItem("Zoom Out",    imageWindow,    SLOT(zoomout()), Key_X);

QPopupMenu *categorymenu = new QPopupMenu(this);
   categorymenu->insertItem("None",     this, SLOT(none()));
   categorymenu->insertItem("Faces",    this, SLOT(faces()));
   categorymenu->insertItem("Vehicles", this, SLOT(vehicles()));
 
//QMenuBar *m = new QMenuBar( this );
 QMenuBar *m = menuBar(); 
   m->setSeparator( QMenuBar::InWindowsStyle );
   m->insertItem("&File", filemenu ); // stick the menu on the bar
   m->insertItem("&Select", selectmenu );
   m->insertItem("&Next", nextmenu );
   m->insertItem("&Edit", editmenu );
   m->insertItem("&View", viewmenu );
   m->insertItem("&Option", optionmenu );
   m->insertItem("&Classify", categorymenu );
 //  mainLayout-> setMenuBar( m );
 QToolBar *t = new QToolBar( this, "selectionOperations" );
   t->setLabel( "Selection Functions" );
   QToolButton * selectPoint =
        new QToolButton(QIconSet(QPixmap("point.bmp")) , "Select Point", QString::null, imageWindow, SLOT( setModePoint()), t, "select point");
   QToolButton * selectRect =
        new QToolButton(QIconSet(QPixmap("rect.bmp")) ,  "Select Rect",  QString::null, imageWindow, SLOT( setModeRect()),  t, "select rect");
   QToolButton * selectFixedRect =
        new QToolButton(QIconSet(QPixmap("fixedrect.bmp")) ,  "Select Fixed Rect",  QString::null, imageWindow, SLOT( setModeFixedRect()),  t, "select fixed rect");

   QSpinBox * w = new QSpinBox( t, "widthSelector" );
   connect( w, SIGNAL( valueChanged(int)), imageWindow, SLOT( setFixedWidth( int )));
   w->setPrefix("W: ");
   w->setMaxValue(10000);
   w->setMinValue(2);
   w->setValue(32);
   QSpinBox * h = new QSpinBox( t, "heightSelector" );
   connect( h, SIGNAL( valueChanged(int)), imageWindow, SLOT( setFixedHeight( int )));
   h->setMinValue(2);
   h->setMaxValue(10000);
   h->setPrefix("H: ");
   h->setValue(32);

   t->show();

}

/** adds every file in a directory to the map of images **/
void Window::openDir()
{
	QString dirName = QFileDialog::
	getExistingDirectory( QString :: null //current dir to start
				, this 
				, "directory with the images?"
				, "pick a directory"
				, TRUE );
	if( dirName != QString::null) 
		emit(openingDir( dirName ) );
}											

void Window::open()
{ 
	//open the dialog to grab the file name
  QStringList fileNameOpen = QFileDialog::
    getOpenFileNames( "Images (*.jpg *.xpm *.bmp *.bz2 *.tif *.tiff *.pgm *.ppm)", NULL,  
		     this,
		     "openfilesdialog"		   
		     "Choose images to open");  
  //broadcast an open this file command
 if( fileNameOpen.count() > 0 )
	  emit(openFrom(fileNameOpen));
}

void Window::openListOfImages()
{ 
	//open the dialog to grab the file name
  QString filename = QFileDialog::
    getOpenFileName( "Dat Files (*.dat)", NULL,
		     this,
		     "openfiledialog"		   
		     "Choose file with list of images ");  
  if( filename != QString::null) 
 	 emit(openImageList( filename ) );
}
		  
void Window::none()
{
    vehiclesWidget->hide();
    facesWidget->hide();
}
//displays the set of buttons for faces mode
void Window:: faces()
{
  vehiclesWidget->hide();
  facesWidget->show();
}

//displays the set of buttons for vehicles mode
void Window:: vehicles()
{
  facesWidget->hide();
  vehiclesWidget->show();
}

// setButton is not a slot its a function so you cant connect straight into it
void Window:: setData( int a, int b){
	switch( a ){
		case( 0 ) : setAge                              ( b ) ; break;
		case( 1 ) : setExpression                       ( b ) ; break;
		case( 2 ) : setGender                           ( b ) ; break;
		case( 3 ) : setGlasses                          ( b ) ; break;
		case( 4 ) : setAngle                            ( b ) ; break;
		case( 5 ) : setRace                             ( b ) ; break;
		case( 6 ) : setBrightness                       ( b ) ; break;
		case( 7 ) : setComplexity                       ( b ) ; break;
		case( 8 ) : setType                             ( b ) ; break;
	}
}

void Window:: setAge(int a)        { facesWidget->ageButtonGroup          ->setButton(a);}
void Window:: setExpression(int a) { facesWidget->expressionButtonGroup   ->setButton(a);}
void Window:: setGender(int a)     { facesWidget->genderButtonGroup       ->setButton(a);}
void Window:: setGlasses(int a)    { facesWidget->glassesButtonGroup      ->setButton(a);}
void Window:: setAngle(int a)      { facesWidget->angleButtonGroup        ->setButton(a);}
void Window:: setRace(int a)       { facesWidget->raceButtonGroup         ->setButton(a);}
void Window:: setBrightness(int a) { vehiclesWidget->brightnessButtonGroup->setButton(a);}
void Window:: setComplexity(int a) { vehiclesWidget->complexityButtonGroup->setButton(a);}
void Window:: setType(int a)       { vehiclesWidget->typeButtonGroup      ->setButton(a);}

void Window::aboutToQuit()
{
	statManager->saveAll();
}
	
